from argparse import ArgumentParser
from .draw import draw
from .parser import UMLVizParser
import sys

def main():
    parser = ArgumentParser(prog='umlviz', description='Generates a UML diagram based on C-like language')
    parser.add_argument('name', help='The name of the UML diagram')
    parser.add_argument('format', help='The output format')
    parser.add_argument('input', nargs='+', help='The input file(s)')
    args = parser.parse_args()
    parser = UMLVizParser()
    ast = []
    for input in args.input:
        with open(input, 'r') as f:
            ast.extend(parser.parse(f.read()))
    draw(args.name, args.format, ast)




# parsetab.py
# This file is automatically generated. Do not edit.
# pylint: disable=W,C,R
_tabversion = '3.10'

_lr_method = 'LALR'

_lr_signature = 'CLOSE_CURLY COLON ENUM METHOD NEWLINE OBJECT OPEN_CURLY WORDmodel : type\n| model type\n| model NEWLINEtype : objectobject : OBJECT WORD OPEN_CURLY NEWLINE members NEWLINE CLOSE_CURLYmembers : member\n| members NEWLINE membermember : variable\n| methodvariable : WORD WORDmethod : METHOD'
    
_lr_action_items = {'OBJECT':([0,1,2,3,5,6,18,],[4,4,-1,-4,-2,-3,-5,]),'$end':([1,2,3,5,6,18,],[0,-1,-4,-2,-3,-5,]),'NEWLINE':([1,2,3,5,6,8,11,12,13,14,15,16,18,19,],[6,-1,-4,-2,-3,9,17,-6,-8,-9,-11,-10,-5,-7,]),'WORD':([4,9,10,17,],[7,10,16,10,]),'OPEN_CURLY':([7,],[8,]),'METHOD':([9,17,],[15,15,]),'CLOSE_CURLY':([17,],[18,]),}

_lr_action = {}
for _k, _v in _lr_action_items.items():
   for _x,_y in zip(_v[0],_v[1]):
      if not _x in _lr_action:  _lr_action[_x] = {}
      _lr_action[_x][_k] = _y
del _lr_action_items

_lr_goto_items = {'model':([0,],[1,]),'type':([0,1,],[2,5,]),'object':([0,1,],[3,3,]),'members':([9,],[11,]),'member':([9,17,],[12,19,]),'variable':([9,17,],[13,13,]),'method':([9,17,],[14,14,]),}

_lr_goto = {}
for _k, _v in _lr_goto_items.items():
   for _x, _y in zip(_v[0], _v[1]):
       if not _x in _lr_goto: _lr_goto[_x] = {}
       _lr_goto[_x][_k] = _y
del _lr_goto_items
_lr_productions = [
  ("S' -> model","S'",1,None,None,None),
  ('model -> type','model',1,'p_model','parser.py',7),
  ('model -> model type','model',2,'p_model','parser.py',8),
  ('model -> model NEWLINE','model',2,'p_model','parser.py',9),
  ('type -> object','type',1,'p_type_object','parser.py',20),
  ('object -> OBJECT WORD OPEN_CURLY NEWLINE members NEWLINE CLOSE_CURLY','object',7,'p_object','parser.py',24),
  ('members -> member','members',1,'p_members','parser.py',31),
  ('members -> members NEWLINE member','members',3,'p_members','parser.py',32),
  ('member -> variable','member',1,'p_member','parser.py',40),
  ('member -> method','member',1,'p_member','parser.py',41),
  ('variable -> WORD WORD','variable',2,'p_variable','parser.py',45),
  ('method -> METHOD','method',1,'p_method','parser.py',53),
]



import ply.yacc as yacc # type: ignore[import-untyped]
from .lexer import UMLVizLexer

class UMLVizParser:

    def p_model(self, p):
        '''model : type
                 | model type
                 | model NEWLINE'''


        if len(p) == 2:
            p[0] = [p[1]]
        elif len(p) == 3 and isinstance(p[1], dict): 
            p[0] = p[1].append(p[2])
        elif len(p) == 3:
            p[0] = p[1]

    def p_type_object(self, p):
        'type : object'
        p[0] = p[1]

    def p_object(self, p):
        'object : OBJECT WORD OPEN_CURLY NEWLINE members NEWLINE CLOSE_CURLY'
        p[0] = {
            'name' : p[2],
            'members': p[5],
        }

    def p_members(self, p):
        '''members : member
                   | members NEWLINE member'''
        if len(p) == 2:
            p[0] = [p[1]]
        elif len(p) == 4:
            p[1].append(p[3])
            p[0] = p[1]

    def p_member(self, p):
        '''member : variable
                  | method'''
        p[0] = p[1]

    def p_variable(self, p):
        'variable : WORD WORD'
        p[0] = {
            'declaration': 'variable',
            'type': p[1],
            'name': p[2]
        }
    
    def p_method(self, p):
        'method : METHOD'
        p[0] = {
            'declaration': 'method',
            'name': p[1]
        }

    def p_error(self, p):
        raise SyntaxError("Invalid syntax.")

    def __init__(self,**kwargs):
        self.tokens = UMLVizLexer.tokens
        self.parser = yacc.yacc(module=self, **kwargs)

    def parse(self, data, debug=False):
        return self.parser.parse(data, debug=debug, lexer=UMLVizLexer().lexer)




import ply.lex as lex # type: ignore[import-untyped]

class UMLVizLexer:

    reserved = {
        'class': 'OBJECT',
        'struct': 'OBJECT',
        'enum': 'ENUM',
    }

    tokens = [
        'OBJECT',
        'ENUM',
        'METHOD',
        'WORD',
        'COLON',
        'OPEN_CURLY',
        'CLOSE_CURLY',
        'NEWLINE',
    ]

    def t_METHOD(self, t):
        r'[a-zA-Z]([a-zA-Z0-9_]|::)+\(\)'
        return t

    def t_WORD(self, t):
        r'[a-zA-Z]([a-zA-Z0-9_]|::)+'
        t.type = self.reserved.get(t.value, 'WORD')
        return t

    t_COLON = r':'
    t_OPEN_CURLY = r'{'
    t_CLOSE_CURLY = r'}'
    t_NEWLINE = r'\n+'

    t_ignore  = ' \t'

    def t_error(self, t):
        raise SyntaxError("Illegal character '%s' found." % (t.value[0]))

    def __init__(self, **kwargs):
        self.lexer = lex.lex(module=self, **kwargs)

    def lex(self, data):
        self.lexer.input(data)
        token = self.lexer.token()
        result = []
        while token is not None:
            result.append(token)
            token = self.lexer.token()
        return result



from graphviz import Graph # type: ignore[import-untyped]

def draw(name, format, ast):
    g = Graph(name=name)
    g.attr('node', shape='record')
    for node in ast:
        label = "{%s" % (node['name'])
        label += "|"
        member_vars = filter(lambda x : x['declaration'] == 'variable', node['members'])
        for member in member_vars:
            label += "+ %s : %s\l" % (member['name'], member['type'])
        label += "|"
        member_funcs = filter(lambda x : x['declaration'] == 'method', node['members'])
        for member in member_funcs:
            label += "+ %s\l" % (member['name'])
        label += "}"
        g.node(node['name'], label)
    g.render(filename=name, format=format, cleanup=True)



from umlviz.parser import UMLVizParser
import unittest


class TestLexer(unittest.TestCase):

    def test_class(self):
        input_string = """class MyClass {
        int m_member1
        bool m_member2
        }"""

        parser = UMLVizParser()
        result = parser.parse(input_string)
        assert len(result) == 1
        assert 'name' in result[0] and result[0]['name'] == 'MyClass'
        assert 'members' in result[0] and isinstance(result[0]['members'], list)
        assert 'declaration' in result[0]['members'][0] and result[0]['members'][0]['declaration'] == 'variable'
        assert 'type' in result[0]['members'][0] and result[0]['members'][0]['type'] == 'int'
        assert 'name' in result[0]['members'][0] and result[0]['members'][0]['name'] == 'm_member1'
        assert 'declaration' in result[0]['members'][1] and result[0]['members'][1]['declaration'] == 'variable'
        assert 'type' in result[0]['members'][1] and result[0]['members'][1]['type'] == 'bool'
        assert 'name' in result[0]['members'][1] and result[0]['members'][1]['name'] == 'm_member2'



from umlviz.lexer import UMLVizLexer
import unittest


class TestLexer(unittest.TestCase):

    def test_class(self):
        input_string = """class MyClass {
        int m_member1
        std::string m_member2
        }"""

        lexer = UMLVizLexer()
        result = lexer.lex(input_string)
        assert result[0].type  == "OBJECT"
        assert result[0].value == "class"
        assert result[1].type  == "WORD"
        assert result[1].value == "MyClass"
        assert result[2].type  == "OPEN_CURLY"
        assert result[2].value == "{"
        assert result[3].type  == "NEWLINE"
        assert result[3].value == "\n"
        assert result[4].type  == "WORD"
        assert result[4].value == "int"
        assert result[5].type  == "WORD"
        assert result[5].value == "m_member1"
        assert result[6].type  == "NEWLINE"
        assert result[6].value == "\n"
        assert result[7].type  == "WORD"
        assert result[7].value == "std::string"
        assert result[8].type  == "WORD"
        assert result[8].value == "m_member2"
        assert result[9].type  == "NEWLINE"
        assert result[9].value == "\n"
        assert result[10].type  == "CLOSE_CURLY"
        assert result[10].value == "}"
        assert len(result) == 11
